﻿Imports System.ComponentModel
Imports System.Xml.Serialization

<SerializableAttribute()> _
<TypeConverterAttribute(GetType(MyMarginsConverter))> _
Public Class MyMargins
    Implements ICloneable

    Public Enum Units
        HundredsOfAnInch = 0
        TensOfAMillimeter = 1
    End Enum

    Private _Left As Single
    Private _Right As Single
    Private _Top As Single
    Private _Bottom As Single
    Private _Unit As Units = Units.HundredsOfAnInch

    Public Sub New()
    End Sub

    Public Sub New(ByVal left As Single, ByVal right As Single, _
                   ByVal top As Single, ByVal bottom As Single)
        _Left = Math.Abs(left)
        _Right = Math.Abs(right)
        _Top = Math.Abs(top)
        _Bottom = Math.Abs(bottom)
    End Sub

    Public Sub New(ByVal margins As Printing.Margins)
        _Left = margins.Left
        _Right = margins.Right
        _Top = margins.Top
        _Bottom = margins.Bottom
    End Sub

    Public Property Left() As Single
        Get
            Return _Left
        End Get
        Set(ByVal value As Single)
            _Left = Math.Abs(value)
        End Set
    End Property

    Public Property Right() As Single
        Get
            Return _Right
        End Get
        Set(ByVal value As Single)
            _Right = Math.Abs(value)
        End Set
    End Property

    Public Property Top() As Single
        Get
            Return _Top
        End Get
        Set(ByVal value As Single)
            _Top = Math.Abs(value)
        End Set
    End Property

    Public Property Bottom() As Single
        Get
            Return _Bottom
        End Get
        Set(ByVal value As Single)
            _Bottom = Math.Abs(value)
        End Set
    End Property

    <xmlignore()> _
    Public Property LeftMetric() As Single
        Get
            Return InchToMetric(_Left)
        End Get
        Set(ByVal value As Single)
            _Left = MetricToInch(Math.Abs(value))
        End Set
    End Property

    <XmlIgnore()> _
    Public Property RightMetric() As Single
        Get
            Return InchToMetric(_Right)
        End Get
        Set(ByVal value As Single)
            _Right = MetricToInch(Math.Abs(value))
        End Set
    End Property

    <XmlIgnore()> _
    Public Property TopMetric() As Single
        Get
            Return InchToMetric(_Top)
        End Get
        Set(ByVal value As Single)
            _Top = MetricToInch(Math.Abs(value))
        End Set
    End Property

    <XmlIgnore()> _
    Public Property BottomMetric() As Single
        Get
            Return InchToMetric(_Bottom)
        End Get
        Set(ByVal value As Single)
            _Bottom = MetricToInch(Math.Abs(value))
        End Set
    End Property

    Private Function MetricToInch(ByVal value As Single) As Single
        Return CSng(Math.Round(value * 3.937 / 10, 2))
    End Function

    Private Function InchToMetric(ByVal value As Single) As Single
        Return CSng(Math.Round(value / 3.937 * 10, 2))
    End Function

    Public Function Clone() As Object Implements System.ICloneable.Clone
        Dim cln As New MyMargins
        cln.Left = _Left
        cln.Right = _Right
        cln.Top = _Top
        cln.Bottom = _Bottom
        Return cln
    End Function

    Public Overrides Function Equals(ByVal obj As Object) As Boolean
        If obj.GetType.Name = "MyMargins" Then
            Dim vo As MyMargins = DirectCast(obj, MyMargins)
            If vo.Left = _Left AndAlso vo.Right = _Right _
                AndAlso vo.Top = _Top AndAlso vo.Bottom = _Bottom Then
                Return True
            End If
        End If
        Return False
    End Function

End Class
