﻿Imports System.ComponentModel

Public Class MyMarginsConverter
    Inherits ExpandableObjectConverter

    Public Sub New()
        MyBase.new()
    End Sub

  Public Overrides Function CanConvertFrom (_
            ByVal context As System.ComponentModel.ITypeDescriptorContext, _
                ByVal sourceType As System.Type) As Boolean

        If sourceType Is GetType(MyMargins) Then
            Return True
        ElseIf sourceType Is GetType(String) Then
            Return True
        Else
            Return MyBase.CanConvertFrom(sourceType)
        End If
    End Function

    Public Overrides Function CanConvertTo( _
            ByVal context As System.ComponentModel.ITypeDescriptorContext, _
            ByVal destinationType As System.Type) As Boolean

        If destinationType Is GetType(String) Then
            Return True
        End If
    End Function

    Public Overrides Function ConvertFrom( _
            ByVal context As System.ComponentModel.ITypeDescriptorContext, _
            ByVal culture As System.Globalization.CultureInfo, _
            ByVal value As Object) As Object

        If TypeOf value Is Printing.Margins Then
            Return New MyMargins(DirectCast(value, Printing.Margins))
        ElseIf TypeOf value Is String Then
            Return ObjectFromString(DirectCast(value, String), culture)
        End If
        Return MyBase.ConvertFrom(context, culture, value)
    End Function

    Public Overrides Function ConvertTo( _
            ByVal context As System.ComponentModel.ITypeDescriptorContext, _
            ByVal culture As System.Globalization.CultureInfo, _
            ByVal value As Object, _
            ByVal destinationType As System.Type) As Object

        If destinationType Is GetType(String) Then
            Dim src As MyMargins = DirectCast(value, MyMargins)
            Dim retval As String = "{MyMargins"
            retval &= " Left=" & src.Left.ToString("g", culture)
            retval &= " Right=" & src.Right.ToString("g", culture)
            retval &= " Top=" & src.Top.ToString("g", culture)
            retval &= " Bottom=" & src.Bottom.ToString("g", culture)
            Return retval
        End If
        Return MyBase.ConvertTo(context, culture, value, destinationType)
    End Function

    Public Overloads Function IsValid(ByVal value As Object) As Boolean
        If TypeOf value Is String Then
            Dim v As String = DirectCast(value, String)
            Return v.StartsWith("{MyMargins") And _
                v.Contains("Left=") And _
                v.Contains("Right=") And _
                v.Contains("Top=") And _
                v.Contains("Bottom=")
        ElseIf TypeOf value Is Printing.Margins Then
            Return True
        Else
            Return MyBase.IsValid(value)
        End If
    End Function

    Public Overrides Function IsValid( _
            ByVal context As System.ComponentModel.ITypeDescriptorContext, _
            ByVal value As Object) As Boolean

        Return IsValid(value)
    End Function

    Private Function ObjectFromString(ByVal value As String, _
            ByVal culture As Globalization.CultureInfo) As MyMargins

        Dim v As String() = _
            value.Replace("{", "").Replace("}", "").Split(New Char() {" "c})
        Dim mm As New MyMargins
        If v(0).Trim <> "MyMargins" Then
            Return Nothing
        End If
        For Each s As String In v
            If s.StartsWith("Left=") Then
                Single.TryParse(s.Substring(s.IndexOf("=") + 1), _
                    System.Globalization.NumberStyles.AllowDecimalPoint, _
                    culture, mm.Left)
            End If
            If s.StartsWith("Right=") Then
                Single.TryParse(s.Substring(s.IndexOf("=") + 1), _
                    System.Globalization.NumberStyles.AllowDecimalPoint, _
                    culture, mm.Right)
            End If
            If s.StartsWith("Top=") Then
                Single.TryParse(s.Substring(s.IndexOf("=") + 1), _
                    System.Globalization.NumberStyles.AllowDecimalPoint, _
                    culture, mm.Top)
            End If
            If s.StartsWith("Bottom=") Then
                Single.TryParse(s.Substring(s.IndexOf("=") + 1), _
                    System.Globalization.NumberStyles.AllowDecimalPoint, _
                    culture, mm.Bottom)
            End If
        Next
        Return mm
    End Function
End Class

