﻿Imports System.Drawing.Printing

Public Class frmPSDBug

    Private _PageSettings As Printing.PageSettings
    Private _PrinterSettings As PrinterSettings
    Private _myPageSettings As MyPageSettings

    Private Sub Form1_Load(ByVal sender As Object, _
                           ByVal e As System.EventArgs) Handles Me.Load
        _PrinterSettings = GetDefaultPrinterSettings()
        _PageSettings = _PrinterSettings.DefaultPageSettings
        _myPageSettings = New MyPageSettings(_PrinterSettings)
    End Sub

    Private Sub btnPageSetup_Click(ByVal sender As Object, _
                                   ByVal e As System.EventArgs) _
                               Handles btnPageSetup.Click

        If rbtOwnDialog.Checked Then
            Dim pdlg As New MyPageSetupDialog
            pdlg.EnableMetric = ckbMetric.Checked
            pdlg.PageSettings = _myPageSettings
            ShowMyMargins(pdlg.PageSettings.Margins, False)
            If pdlg.ShowDialog() = Windows.Forms.DialogResult.OK Then
                _myPageSettings = pdlg.PageSettings
            End If
            ShowMyMargins(_myPageSettings.Margins, True)
        Else
            Dim pdlg As New PageSetupDialog

            ShowMargins(_PageSettings.Margins, False)

            ' alte Seitenränder merken
            Dim oldMargins As Printing.Margins = _
                DirectCast(_PageSettings.Margins.Clone, Printing.Margins)

            ' mit Korrektur
            If rbtCorrection.Checked Then
                CorrectMargins(_PageSettings)
            End If

            ' PageSetupDialog initialisieren
            pdlg.PrinterSettings = _PrinterSettings
            pdlg.PageSettings = _PageSettings
            pdlg.EnableMetric = ckbMetric.Checked

            ' und los geht's
            If pdlg.ShowDialog() = Windows.Forms.DialogResult.OK Then
                ' Geänderte Seiteneinstellungen übernehmen
                _PageSettings = pdlg.PageSettings
            Else
                ' Bei Abbruch die nicht "korrigierten" Seitenränder wieder 
                '  zurück schreiben
                If rbtCorrection.Checked Then
                    _PageSettings.Margins = oldMargins
                End If
            End If

            ShowMargins(pdlg.PageSettings.Margins, True)
        End If
    End Sub

    Private Sub CorrectMargins(ByRef pgs As Printing.PageSettings)
        If Globalization.RegionInfo.CurrentRegion.IsMetric Then
            ' Wegen Rundungsfehlern darf nicht durch 0.3948 geteilt werden!
            Dim fkt As Single = 3.948
            pgs.Margins.Left = CInt(Math.Round(pgs.Margins.Left / fkt) * 10)
            pgs.Margins.Top = CInt(Math.Round(pgs.Margins.Top / fkt) * 10)
            pgs.Margins.Right = CInt(Math.Round(pgs.Margins.Right / fkt) * 10)
            pgs.Margins.Bottom = CInt(Math.Round(pgs.Margins.Bottom / fkt) * 10)
        End If
    End Sub

    Private Function GetDefaultPrinterSettings() As Printing.PrinterSettings
        Dim prs As New Printing.PrinterSettings
        For Each prtn As String In Printing.PrinterSettings.InstalledPrinters
            prs.PrinterName = prtn
            If prs.IsDefaultPrinter Then
                Return prs
            End If
        Next
        Return Nothing
    End Function

    Private Sub ShowMargins(ByVal vMargins As Printing.Margins, _
                            ByVal after As Boolean)
        If after Then
            lblALeft.Text = vMargins.Left.ToString()
            lblATop.Text = vMargins.Top.ToString()
            lblARight.Text = vMargins.Right.ToString()
            lblABottom.Text = vMargins.Bottom.ToString()
        Else
            lblBLeft.Text = vMargins.Left.ToString()
            lblBTop.Text = vMargins.Top.ToString()
            lblBRight.Text = vMargins.Right.ToString()
            lblBBottom.Text = vMargins.Bottom.ToString()
        End If
    End Sub

    Private Sub ShowMyMargins(ByVal vMargins As MyMargins, _
                              ByVal after As Boolean)
        If after Then
            lblALeft.Text = vMargins.Left.ToString()
            lblATop.Text = vMargins.Top.ToString()
            lblARight.Text = vMargins.Right.ToString()
            lblABottom.Text = vMargins.Bottom.ToString()
        Else
            lblBLeft.Text = vMargins.Left.ToString()
            lblBTop.Text = vMargins.Top.ToString()
            lblBRight.Text = vMargins.Right.ToString()
            lblBBottom.Text = vMargins.Bottom.ToString()
        End If
    End Sub

    Private Sub rbtCorrection_CheckedChanged(ByVal sender As Object, _
                                             ByVal e As System.EventArgs) _
                                         Handles rbtCorrection.CheckedChanged
        If rbtCorrection.Checked Then
            ckbMetric.Checked = False
            ckbMetric.Enabled = False
        Else
            ckbMetric.Enabled = True
        End If
    End Sub
End Class